export default ( { props } ) => {
    if ( ! props ) return '';

    const { __ } = wp.i18n;
    const { 
        PanelBody, 
        TextControl,
        ToggleControl,
        RangeControl,
        RadioControl,
    } = wp.components;
    const { 
        setAttributes,
        attributes: {
            postStyle,
            showExcerpt,
            excerptType,
            excerptLength,
            showReadMore,
            readMoreText,
        }
	} = props;

    return(
        <PanelBody 
            title={ __( 'Post Content Settings', 'sv_posts' ) }
            initialOpen={ false }
        >
            <ToggleControl
                label={ __( 'Post Content', 'sv_posts' ) }
                checked={ showExcerpt }
                onChange={ () => setAttributes( { showExcerpt: ! showExcerpt } ) }
            />
            {
                showExcerpt ?
                <RadioControl
                    label={ __( 'Show', 'sv_posts' ) }
                    selected={ excerptType }
                    options={ [
                        { label: __( 'Excerpt', 'sv_posts' ), value: 'excerpt' },
                        { label: __( 'Full Post', 'sv_posts' ), value: 'full_post' },
                    ] }
                    onChange={ ( value ) => setAttributes( { excerptType: value } ) }
                /> :
                null
            }
            {
                showExcerpt && excerptType === 'excerpt' ?
                    <RangeControl
                        label={ __( 'Max number of words in excerpt', 'sv_posts' ) }
                        value={ excerptLength }
                        onChange={ ( value ) => setAttributes( { excerptLength: value } ) }
                        min={ 10 }
                        max={ 100 }
                    />
                : null
            }
            {
                showExcerpt && excerptType === 'excerpt' && postStyle !== 'tile' ? [
                    <ToggleControl
                        label={ __( 'Show Read More', 'sv_posts' ) }
                        checked={ showReadMore }
                        onChange={ () => setAttributes( { showReadMore: ! showReadMore } ) }
                    />,
                    showReadMore ? 
                        <TextControl
                            label={ __( 'Read More Text', 'sv_posts' ) }
                            value={ readMoreText }
                            onChange={ ( value ) => setAttributes( { readMoreText: value } ) }
                        />
                    : null
                ]
                : null
            }
        </PanelBody>
    );
}