export default ( { props } ) => {
    if ( ! props ) return '';

    const { __ } = wp.i18n;
    const { 
        PanelBody, 
        ToggleControl,
        SelectControl,
    } = wp.components;
    const { 
        setAttributes,
        attributes: {
            showTitle,
            showThumbnail,
            thumbnailSize,
        }
    } = props;

    return(
        <PanelBody 
            title={ __( 'Post Settings', 'sv_posts' ) }
            initialOpen={ false }
        >
            <ToggleControl
                label={ __( 'Display Title', 'sv_posts' ) }
                checked={ showTitle }
                onChange={ () => setAttributes( { showTitle: ! showTitle } ) }
            />
            <ToggleControl
                label={ __( 'Display Thumbnail', 'sv_posts' ) }
                checked={ showThumbnail }
                onChange={ () => setAttributes( { showThumbnail: ! showThumbnail } ) }
            />
            {
                showThumbnail ?
                <SelectControl 
                    label={ __( 'Thumbnail Size', 'sv_posts' ) }
                    value={ thumbnailSize }
                    options={ [
                        { label: __( 'Thumbnail', 'sv_posts' ), value: 'thumbnail' },
                        { label: __( 'Medium', 'sv_posts' ), value: 'medium' },
                        { label: __( 'Medium Large', 'sv_posts' ), value: 'medium_large' },
                        { label: __( 'Large', 'sv_posts' ), value: 'large' },
                        { label: __( 'Full Size', 'sv_posts' ), value: 'full' },
                    ] }
                    onChange={ ( value ) => setAttributes( { thumbnailSize: value } ) }
                /> 
                : null
            }
        </PanelBody>
    );
}