import List from './post_styles/style_list';
import Card from './post_styles/style_card';
import Tile from './post_styles/style_tile';

export default ( { props, data } ) => {
    const { __ } = wp.i18n;
    const { Placeholder } = wp.components;

    if ( ! props || ! data ) return <Placeholder label={ __( 'Missing information!', 'sv_posts' ) } />;

    const { 
        className,
        attributes: { 
            // Style & Layout
            postStyle,
            layout,
            columns,
            align,
        }
    } = props;

    let classes = className + ' post-style-' + postStyle;

    if ( postStyle === 'card' ) {
        classes += ' layout-' + layout;
    }

    if ( postStyle !== 'columns' ) {
        classes += ' layout-cols-' + columns;
    }

    if ( align ) {
        classes += ' align' + align;
    }

    return(
        <div className={ classes }>
        {
            data.posts.map( post => {
                switch( postStyle ) {
                    case 'list':    return ( <List props={ props } data={ data } post={ post } /> );
                    case 'card':    return ( <Card props={ props } data={ data } post={ post } /> );
                    case 'tile':    return ( <Tile props={ props } data={ data } post={ post } /> );
                }
            } ) 
        }
        </div>
    );
}