import user from '../../icons/user';
import clock from '../../icons/clock';

export default ( { props, data, post } ) => {
    if ( ! props || ! post ) return '';

    const { 
        attributes: {
            // Thumbnail Settings
            showThumbnail,
            thumbnailSize,

            // Content Settings
            showTitle,
            showAuthor,
            showDate,
            showCategories,

            // Excerpt Settings
            showExcerpt,
            excerptType,
            excerptLength,

            // Color Settings
            backgroundColor,
            backgroundColorClass,
            titleColor,
            titleColorClass,
            excerptColor,
            excerptColorClass,
            infoColor,
            infoColorClass,
            infoIconColor,
            infoIconColorClass,
            categoryColor,
            categoryColorClass,
            categoryBackgroundColor,
            categoryBackgroundColorClass,
        }
    } = props;

    // Returns the post thumbnail as img element
    function getThumbnail() {
        if ( ! post.featured_media ) return '';

        const media = wp.data.select('core').getMedia( post.featured_media );

        if ( media && media.media_details.sizes && media.media_details.sizes[ thumbnailSize ] ) {
            const size      = media.media_details.sizes[ thumbnailSize ];
            const classes   = 'attachment-' + thumbnailSize + ' size-' + thumbnailSize + ' wp-post-image';
            const img       = <img width={ size.width } height={ size.height } src={ size.source_url } className={ classes } />;

            return img;
        }
    }

    // Returns the post categories
    function getCategories() {
        if ( ! post.categories || post.categories.length < 1 ) return '';

        return post.categories.map( id => {
            let category = data.categories.find( cat => {
                return cat.id === id;
            } )
            
            if ( category && category.link ) {
                return <a href='#' 
                        className={[ categoryColorClass, categoryBackgroundColorClass ]} 
                        style={{ color: categoryColor, backgroundColor: categoryBackgroundColor }}>
                        { category.name }</a>;
            }
        } );
    }

    // Returns the post author
    function getAuthor() {
        if ( ! data.authors || data.authors.length < 1 ) return '';

        let author = data.authors.find( author => {
            return author.id === post.author;
        } )

        return <a href='#' className={ infoColorClass } style={{ color: infoColor }}>{ author.name }</a>;
    }

    // Returns the date in the local date format
    function getFormatedDate() {
        return new Date(post.date).toLocaleDateString();
    }

    // Classes
    const hasPostThumbnail = post.featured_media ? 'has-post-thumbnail' : '';
    let classes = [
        post.type,
        'type-' + post.type,
        'status-' + post.status,
        'format-' + post.format,
        hasPostThumbnail,
        backgroundColorClass,
    ];

    // Excerpt
    let excerpt                 = post.excerpt.rendered;
    const excerptElement        = document.createElement( 'div' );
    excerptElement.innerHTML    = excerpt;
    excerpt                     = excerptElement.textContent || excerptElement.innerText || '';

    let fullPost                = post.content.rendered;
    const fullPostElement       = document.createElement( 'div' );
    fullPostElement.innerHTML   = fullPost;
    fullPost                    = fullPostElement.textContent || fullPostElement.innerText || '';

    return(
        <article id={ 'post-' + post.id } className={ classes.join(' ') } style={{ backgroundColor: backgroundColor }}>
            <div className="thumbnail">
                {   
                    // Categories
                    showCategories && post.categories && post.categories.length > 0 ? 
                    <div className="categories">
                        {
                            data.categories ? getCategories() : null
                        }
                    </div>
                    : null
                }
                {
                    showThumbnail && post.featured_media ?
                    <a href='#'>{ getThumbnail() }</a>
                    : null
                }
            </div>
            <div className="wrapper">
                <div className="info">
                { 
                    // Title
                    showTitle ? 
                    <div className="title">
                        <a href='#' className={ titleColorClass } style={{ color: titleColor }}>
                            {post.title.rendered}
                        </a>
                    </div>
                    : null
                }
                {
                    // Author
                    showAuthor ?
                    <div className="author">
                        <i className={ infoIconColorClass } style={{ color: infoIconColor }}>{ user }</i>
                        { getAuthor() }
                    </div>
                    : null
                }
                {
                    // Date
                    showDate ? 
                    <div className="date">
                        <i className={ infoIconColorClass } style={{ color: infoIconColor }}>{ clock }</i>
                        <a href="" className={ infoColorClass } style={{ color: infoColor }}>{ getFormatedDate() }</a>
                    </div>
                    : null
                }
                </div>
                {
                    // Excerpt
                    showExcerpt ?
                    <div className="excerpt">
                        <p className={ excerptColorClass } style={{ color: excerptColor }}>
                        { 
                            excerptType === 'excerpt' ?
                            excerptLength < excerpt.trim().split( ' ' ).length ?
                            excerpt.trim().split( ' ', excerptLength ).join( ' ' ) + ' ...':
                            excerpt.trim().split( ' ', excerptLength ).join( ' ' ) :
                            fullPost.trim()
                        }
                        </p>
                    </div>
                    : null
                }
            </div>
        </article>
    );
}