import './editor.scss';
import icon from './icons/block';
import edit from './edit';

const { __ } = wp.i18n;
const { registerBlockType } = wp.blocks

registerBlockType( 'straightvisions/sv-posts', {
	title: __( 'SV Posts' ),
	description: __( 'Display your most recent posts, in the way you want it.', 'sv_posts' ),
	icon,
	category: 'straightvisions',
	keywords: [
		__( 'SV Posts', 'sv_posts' ),
		__( 'Blog', 'sv_posts' ),
		__( 'Posts', 'sv_posts' ),
	],
	supports: {
		align:[ 'wide', 'full' ],
	},
	attributes: {
		// List Settings
		postStyle: {
			type: 'string',
			default: 'list',
		},
		layout: {
			type: 'string',
			default: 'masonry',
		},
		columns: {
			type: 'number',
			default: 3,
		},

		// Post Settings
		showTitle: {
			type: 'boolean',
			default: true,
		},
		showThumbnail: {
			type: 'boolean',
			default: true,
		},
		thumbnailSize: {
			type: 'string',
			default: 'medium',
		},

		// Post Meta Settings
		showAuthor: {
			type: 'boolean',
			default: true,
		},
		showDate: {
			type: 'boolean',
			default: true,
		},
		showCategories: {
			type: 'boolean',
			default: true,
		},
		showEdit: {
			type: 'boolean',
			default: true,
		},

		// Post Content Settings
		showExcerpt: {
			type: 'boolean',
			default: true,
		},
		excerptType: {
			type: 'string',
			default: 'excerpt',
		},
		excerptLength: {
			type: 'number',
			default: 15,
		},
		showReadMore: {
			type: 'boolean',
			default: true,
		},
		readMoreText: {
			type: 'string',
			default: __( 'Read more', 'sv_posts' ),
		},

		// Color Settings
		backgroundColor: {
			type: 'string',
		},
		backgroundColorClass: {
			type: 'string',
		},
		titleColor: {
			type: 'string',
		},
		titleColorClass: {
			type: 'string',	
		},
		excerptColor: {
			type: 'string',
		},
		excerptColorClass: {
			type: 'string',
		},
		readMoreColor: {
			type: 'string',
		},
		readMoreColorClass: {
			type: 'string',
		},
		infoColor: {
			type: 'string',
		},
		infoColorClass: {
			type: 'string',
		},
		infoIconColor: {
			type: 'string',
		},
		infoIconColorClass: {
			type: 'string',
		},
		categoryColor: {
			type: 'string',
		},
		categoryColorClass: {
			type: 'string',
		},
		categoryBackgroundColor: {
			type: 'string',
		},
		categoryBackgroundColorClass: {
			type: 'string',
		},

		// Sorting and Filtering
		orderBy: {
			type: 'string',
			default: 'date',
		},
		order: {
			type: 'string',
			default: 'desc',
		},
		postsPerPage: {
			type: 'number',
			default: 5,
		},
		filterByAuthor: {
			type: 'number',
		},
		filterByType: {
			type: 'string',
			default: 'post',
		},
		filterByPosts: {
			type: 'string',
		},
		excludeCurrentPost: {
			type: 'boolean',
			default: 'true',
		},
		filterByThumbnail: {
			type: 'boolean',
			default: false,
		},
		filterByCategories: {
			type: 'boolean',
			default: false,
		},
		filteredCategories: {
			type: 'string',
		},
		filterByDate: {
			type: 'boolean',
			default: false,
		},
		filterByDateStart: {
			type: 'string',
		},
		filterByDateEnd: {
			type: 'string',
		},
		filterByStatus: {
			type: 'boolean',
			default: false,
		},
		filteredStatus: {
			type: 'string',
			default: JSON.stringify( [ 'publish' ] ),
		},

		// Advanced
		className: {
			type: 'string',
		},
	},
	edit,
	save: () => {
		return null;
	}
} );