<?php
namespace sv_posts;

class block extends sv_posts {
	protected $block_attr 	= array();
	protected $icons		= array();

	public function init() {
		$this->init_icons()->register_block();
	}

	public function init_icons(): sv_posts {
		$this->icons['user'] 	= '<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path d="M12 2c2.757 0 5 2.243 5 5.001 0 2.756-2.243 5-5 5s-5-2.244-5-5c0-2.758 2.243-5.001 5-5.001zm0-2c-3.866 0-7 3.134-7 7.001 0 3.865 3.134 7 7 7s7-3.135 7-7c0-3.867-3.134-7.001-7-7.001zm6.369 13.353c-.497.498-1.057.931-1.658 1.302 2.872 1.874 4.378 5.083 4.972 7.346h-19.387c.572-2.29 2.058-5.503 4.973-7.358-.603-.374-1.162-.811-1.658-1.312-4.258 3.072-5.611 8.506-5.611 10.669h24c0-2.142-1.44-7.557-5.631-10.647z"/></svg>';
		$this->icons['clock'] 	= '<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path d="M12 2c5.514 0 10 4.486 10 10s-4.486 10-10 10-10-4.486-10-10 4.486-10 10-10zm0-2c-6.627 0-12 5.373-12 12s5.373 12 12 12 12-5.373 12-12-5.373-12-12-12zm1 12v-6h-2v8h7v-2h-5z"/></svg>';
		$this->icons['archive'] = '<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path d="M21.669 13l-1.385 9h-16.568l-1.385-9h19.338zm2.331-2h-24l2 13h20l2-13zm-20.312-8l-.688-2h17.979l-.604 2h-16.687zm.166 6l-.415-2h17.121l-.39 2h2.03l.8-4h-22l.8 4h2.054zm12.146 7c0-.552-.447-1-1-1h-6c-.553 0-1 .448-1 1s.447 1 1 1h6c.553 0 1-.448 1-1z"/></svg>';
		$this->icons['pen'] 	= '<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path d="M1.439 16.873l-1.439 7.127 7.128-1.437 16.873-16.872-5.69-5.69-16.872 16.872zm4.702 3.848l-3.582.724.721-3.584 2.861 2.86zm15.031-15.032l-13.617 13.618-2.86-2.861 10.825-10.826 2.846 2.846 1.414-1.414-2.846-2.846 1.377-1.377 2.861 2.86z"/></svg>';

		return $this;
	}

	public function render_block( array $attr ): string {
		$this->block_attr = $attr;

		if ( did_action( 'wp_enqueue_scripts' ) ) {
			$this->load_block_assets();
		} else {
			add_action( 'wp_enqueue_scripts', array( $this, 'load_block_assets' ), 999 );
		}

		ob_start();
		
		require( $this->get_path( 'lib/frontend/tpl/style_' . $attr['postStyle'] . '.php' ) );

		$block_content = ob_get_contents();
		ob_end_clean();

		return $block_content;
	}

	public function load_block_assets() {
		if ( ! is_admin() ) {
			$post_style = 'style_' . $this->block_attr['postStyle'];

			$this->get_parent()->get_script( 'common' )->set_is_enqueued();
			$this->get_parent()->get_script( $post_style )->set_is_enqueued();
		}
	}

	private function register_block() {
		register_block_type(
			'straightvisions/sv-posts', array(
				'editor_script' => 'sv-posts-block',
				'editor_style'  => 'sv-posts-block-editor',
				'attributes'	=> array(
					// List Settings
					'postStyle'			=> array(
						'type'		=> 'string',
						'default'	=> 'list',
					),
					'layout'			=> array(
						'type'		=> 'string',
						'default'	=> 'masonry',
					),
					'columns'			=> array(
						'type'		=> 'number',
						'default'	=> 3,
					),

					// Post Settings
					'showTitle'			=> array(
						'type'		=> 'bool',
						'default'	=> true,
					),
					'showThumbnail'		=> array(
						'type'		=> 'bool',
						'default'	=> true,
					),
					'thumbnailSize'		=> array(
						'type'		=> 'string',
						'default'	=> 'medium',
					),

					// Post Meta Settings
					'showAuthor'		=> array(
						'type'		=> 'bool',
						'default'	=> true,
					),
					'showDate'			=> array(
						'type'		=> 'bool',
						'default'	=> true,
					),
					'showCategories'	=> array(
						'type'		=> 'bool',
						'default'	=> true,
					),
					'showEdit'			=> array(
						'type'		=> 'bool',
						'default'	=> true,
					),

					// Post Content Settings
					'showExcerpt'		=> array(
						'type'		=> 'bool',
						'default'	=> true,
					),
					'excerptType'		=> array(
						'type'		=> 'string',
						'default'	=> 'excerpt',
					),
					'excerptLength'		=> array(
						'type'		=> 'number',
						'default'	=> 15,
					),
					'showReadMore'		=> array(
						'type'		=> 'bool',
						'default'	=> true,
					),
					'readMoreText'		=> array(
						'type'		=> 'string',
						'default'	=> __( 'Read more', 'sv_posts' ),
					),

					// Colors Settings
					'backgroundColor'	=> array(
						'type'		=> 'string',
					),
					'backgroundColorClass'=> array(
						'type'		=> 'string',
					),
					'titleColor'		=> array(
						'type'		=> 'string',
					),
					'titleColorClass'	=> array(
						'type'		=> 'string',
					),
					'excerptColor'		=> array(
						'type'		=> 'string',
					),
					'excerptColorClass'	=> array(
						'type'		=> 'string',
					),
					'readMoreColor'		=> array(
						'type'		=> 'string',
					),
					'readMoreColorClass'=> array(
						'type'		=> 'string',
					),
					'infoColor'			=> array(
						'type'		=> 'string',
					),
					'infoColorClass'	=> array(
						'type'		=> 'string',
					),
					'infoIconColor'		=> array(
						'type'		=> 'string',
					),
					'infoIconColorClass'=> array(
						'type'		=> 'string',
					),
					'categoryColor'		=> array(
						'type'		=> 'string',
					),
					'categoryColorClass'=> array(
						'type'		=> 'string',
					),
					'categoryBackgroundColor'		=> array(
						'type'		=> 'string',
					),
					'categoryBackgroundColorClass'	=> array(
						'type'		=> 'string',
					),

					// Sorting and Filtering
					'orderBy'			=> array(
						'type'		=> 'string',
						'default'	=> 'date',
					),
					'order'				=> array(
						'type'		=> 'string',
						'default'	=> 'desc',
					),
					'postsPerPage'		=> array(
						'type'		=> 'number',
						'default'	=> 5,
					),
					'filterByAuthor'	=> array(
						'type'		=> 'number',
					),
					'filterByType'		=> array(
						'type'		=> 'string',
						'default'	=> 'post',
					),
					'filterByPosts'		=> array(
						'type'		=> 'string',
					),
					'excludeCurrentPost'=> array(
						'type'		=> 'bool',
						'default'	=> true,
					),
					'filterByThumbnail'	=> array(
						'type'		=> 'bool',
						'default'	=> false,
					),
					'filterByCategories'=> array(
						'type'		=> 'bool',
						'default'	=> false,
					),
					'filteredCategories'=> array(
						'type'		=> 'string',
					),
					'filterByDate'		=> array(
						'type'		=> 'bool',
						'default'	=> false,
					),
					'filterByDateStart'	=> array(
						'type'		=> 'string',
					),
					'filterByDateEnd'	=> array(
						'type'		=> 'string',
					),
					'filterByStatus'	=> array(
						'type'		=> 'bool',
						'default'	=> false,
					),
					'filteredStatus'	=> array(
						'type'		=> 'string',
						'default'	=> json_encode( array( 'publish' ) ),
					),

					// Advanced
					'className'			=> array(
						'type'		=> 'string',
					),
				),
				'render_callback'	=> array( $this, 'render_block' ),
			)
		);
	}

	// Helper Methods
	// Returns an icon
	public function get_icon( string $icon ): string {
		if ( ! isset( $this->icons[ $icon ] ) ) return '';

		return $this->icons[ $icon ];
	}

	// Returns a string with all classes for the posts wrapper
	public function get_wrapper_class(): string {
		$class 			= array();
		$class[]		= 'wp-block-straightvisions-sv-posts';
		
		// Post Style
		$class[]		= 'post-style-' . $this->block_attr['postStyle'];

		// Layout
		if ( $this->block_attr['postStyle'] === 'card' ) {
			$class[] 	= 'layout-' . $this->block_attr['layout'];
		}

		// Columns
		if ( $this->block_attr['postStyle'] !== 'list' ) {
			$class[] 	= 'layout-cols-' . $this->block_attr['columns'];
		}

		// Alignment
		if ( isset( $this->block_attr['align'] ) ) { 
			$class[] 	= 'align' . $this->block_attr['align'];
		}

		// Additional Classes
		if ( isset( $this->block_attr['className'] ) ) { 
			$class[] 	= $this->block_attr['className'];
		}

		return implode( ' ', $class );
	}

	// Returns a string with all attributes of a single post
	public function get_post_attr(): string {
		$attr 		= array();

		// ID
		$attr[]		= 'id="post-' . get_the_ID() . '"';

		// Class
		$class		= get_post_class();

		if ( 
			isset( $this->block_attr['backgroundColor'] ) 
			&& $this->block_attr['backgroundColorClass'] 
		) {
            $class[] = $this->block_attr['backgroundColorClass'];
		}
		
		if ( ! empty( $class ) ) {
			$attr[]	= 'class="' . implode( ' ', $class ) . '"';
		}

		// Style
		if ( 
			isset( $this->block_attr['backgroundColor'] ) 
			&& ! $this->block_attr['backgroundColorClass'] 
		) {
			$attr[] = 'style="background-color:' . $this->block_attr['backgroundColor'] . '"';
		}


		return implode( ' ', $attr );
	}

	public function get_posts(): array {
		$args = array(
			'posts_per_page'=> $this->block_attr['postsPerPage'],
			'order'			=> $this->block_attr['order'],
			'orderby'		=> $this->block_attr['orderBy'],
			'post_type'		=> $this->block_attr['filterByType'],
		);

		// Show from Author
		if ( ! empty( $this->block_attr['filterByAuthor'] ) ) {
			$args['author'] = $this->block_attr['filterByAuthor'];
		}

		// Filter by Posts
		if ( 
			! empty( $this->block_attr['filterByPosts'] ) 
			&& count( explode( ',', $this->block_attr['filterByPosts'] ) ) > 0 
		) {
			$args['include'] = array_filter( array_map( 'trim', explode( ',', $this->block_attr['filterByPosts'] ) ) );
		}

		// Remove current post from list
		if ( $this->block_attr['excludeCurrentPost'] ) {
			$args['post__not_in'] = array( get_the_ID() );
		}

		// Show only posts with Thumbnail
		if ( $this->block_attr['filterByThumbnail'] ) {
			$args['meta_key'] = '_thumbnail_id';
		}

		// Filter by Cateogories
		if ( 
			$this->block_attr['filterByCategories'] 
			&& $this->block_attr['filteredCategories']
			&& count( json_decode( $this->block_attr['filteredCategories'] ) ) > 0 
			&& $this->block_attr['filterByType'] !== 'page' 
		) {
			$args['category'] = json_decode( $this->block_attr['filteredCategories'] );
		}

		// Filter by Date
		if ( $this->block_attr['filterByDate'] ) {
			$date_start = array(
				'year'	=> date_parse( $this->block_attr['filterByDateStart'] )['year'],
				'month'	=> date_parse( $this->block_attr['filterByDateStart'] )['month'],
				'day'	=> date_parse( $this->block_attr['filterByDateStart'] )['day'],
			);

			$date_end = array(
				'year'	=> date_parse( $this->block_attr['filterByDateEnd'] )['year'],
				'month'	=> date_parse( $this->block_attr['filterByDateEnd'] )['month'],
				'day'	=> date_parse( $this->block_attr['filterByDateEnd'] )['day'],
			);

			$args['date_query']	= array( 
				array( 
					'after' 	=> $date_start, 
					'before' 	=> $date_end, 
					'inclusive' => true, 
				) 
			);
		}

		// Filter by Status
		if ( $this->block_attr['filterByStatus'] ) {
			$args['post_status'] = $this->block_attr['filteredStatus'];
		}

		return get_posts( $args );
	}

	// Returns bool if thumbnail should be displayed
	public function show_thumbnail(): bool {
		if ( 
			isset( $this->block_attr['showThumbnail'] ) 
			&& $this->block_attr['showThumbnail'] 
			&& has_post_thumbnail() 
		) {
			return true;
		}

		return false;
	}

	// Returns bool if title should be displayed
	public function show_title(): bool {
		if ( ! isset( $this->block_attr['showTitle'] ) ) return false;

		return $this->block_attr['showTitle'];
	}

	// Returns a string with all attributes for the post title
	public function get_title_attr(): string {
		$attr 		= array();

		// HREF
		$attr[]		= 'href="' . get_the_permalink() . '"';

		// Class
		$class		= array();

		if ( 
			isset( $this->block_attr['titleColor'] ) 
			&& $this->block_attr['titleColorClass'] 
		) {
            $class[] = $this->block_attr['titleColorClass'];
		}
		
		if ( ! empty( $class ) ) {
			$attr[]	= 'class="' . implode( ' ', $class ) . '"';
		}

		// Style
		if ( 
			isset( $this->block_attr['titleColor'] ) 
			&& ! $this->block_attr['titleColorClass'] 
		) {
			$attr[] = 'style="color:' . $this->block_attr['titleColor'] . '"';
		}


		return implode( ' ', $attr );
	}

	// Returns bool if the excerpt should be displayed
	public function show_excerpt(): bool {
		if ( ! isset( $this->block_attr['showExcerpt'] ) ) return false;

		return $this->block_attr['showExcerpt'];
	}

	// Returns the excerpt or post content as string
	public function get_excerpt(): string {
		$excerpt = '';

		if ( $this->block_attr['excerptType'] === 'full_post' ) {
			$excerpt = strip_tags( get_the_content() );
		} elseif ( str_word_count( get_the_excerpt() ) > 0 ) {
			$excerpt = get_the_excerpt();

			if ( 
				isset( $this->block_attr['excerptLength'] ) 
				&& $this->block_attr['excerptLength'] 
			) {
				$excerpt = wp_trim_words( $excerpt, $this->block_attr['excerptLength'] );
			}
		}

		return $excerpt;
	}

	// Returns a string with all attributes for the post excerpt
	public function get_excerpt_attr(): string {
		$attr 		= array();
		
		// Class
		$class		= array();

		if ( 
			isset( $this->block_attr['excerptColor'] ) 
			&& $this->block_attr['excerptColorClass'] 
		) {
            $class[] = $this->block_attr['excerptColorClass'];
		}
		
		if ( ! empty( $class ) ) {
			$attr[]	= 'class="' . implode( ' ', $class ) . '"';
		}

		// Style
		if ( 
			isset( $this->block_attr['excerptColor'] ) 
			&& ! $this->block_attr['excerptColorClass'] 
		) {
			$attr[] = 'style="color:' . $this->block_attr['excerptColor'] . '"';
		}


		return implode( ' ', $attr );
	}

	// Returns bool if the read more link should be displayed
	public function show_read_more(): bool {
		if ( 
			isset( $this->block_attr['showReadMore'] ) 
			&& $this->block_attr['showReadMore'] 
			&& $this->block_attr['excerptType'] !== 'full_post' 
			&& str_word_count( get_the_excerpt() ) > $this->block_attr['excerptLength'] 
		) {
			return true;
		}

		return false;
	}

	// Returns a string with all attributes for the read more link
	public function get_read_more_attr(): string {
		$attr 		= array();

		// HREF
		$attr[]		= 'href="' . get_the_permalink() . '"';

		// Class
		$class		= array();

		if ( 
			isset( $this->block_attr['readMoreColor'] ) 
			&& $this->block_attr['readMoreColorClass'] 
		) {
            $class[] = $this->block_attr['readMoreColorClass'];
		}
		
		if ( ! empty( $class ) ) {
			$attr[]	= 'class="' . implode( ' ', $class ) . '"';
		}

		// Style
		if ( 
			isset( $this->block_attr['readMoreColor'] ) 
			&& ! $this->block_attr['readMoreColorClass'] 
		) {
			$attr[] = 'style="color:' . $this->block_attr['readMoreColor'] . '"';
		}


		return implode( ' ', $attr );
	}

	// Returns a string with all attributes for the info
	public function get_info_attr(): string {
		$attr 		= array();

		// Class
		$class		= array();

		if ( 
			isset( $this->block_attr['infoColor'] ) 
			&& $this->block_attr['infoColorClass'] 
		) {
            $class[] = $this->block_attr['infoColorClass'];
		}
		
		if ( ! empty( $class ) ) {
			$attr[]	= 'class="' . implode( ' ', $class ) . '"';
		}

		// Style
		if ( 
			isset( $this->block_attr['infoColor'] ) 
			&& ! $this->block_attr['infoColorClass'] 
		) {
			$attr[] = 'style="color:' . $this->block_attr['infoColor'] . '"';
		}


		return implode( ' ', $attr );
	}

	// Returns a string with all attributes for the info icons
	public function get_info_icon_attr(): string {
		$attr 		= array();

		// Class
		$class		= array();

		if ( 
			isset( $this->block_attr['infoIconColor'] ) 
			&& $this->block_attr['infoIconColorClass'] 
		) {
            $class[] = $this->block_attr['infoIconColorClass'];
		}
		
		if ( ! empty( $class ) ) {
			$attr[]	= 'class="' . implode( ' ', $class ) . '"';
		}

		// Style
		if ( 
			isset( $this->block_attr['infoIconColor'] ) 
			&& ! $this->block_attr['infoIconColorClass'] 
		) {
			$attr[] = 'style="color:' . $this->block_attr['infoIconColor'] . '"';
		}


		return implode( ' ', $attr );
	}

	// Returns bool if the author should be displayed
	public function show_author(): bool {
		if ( ! isset( $this->block_attr['showAuthor'] ) ) return false;

		return $this->block_attr['showAuthor'];
	}

	// Returns bool if the date should be displayed
	public function show_date(): bool {
		if ( ! isset( $this->block_attr['showDate'] ) ) return false;

		return $this->block_attr['showDate'];
	}

	// Returns bool if the categories should be displayed
	public function show_categories(): bool {
		if ( ! isset( $this->block_attr['showCategories'] ) ) return false;
		if ( empty( get_the_category() ) ) return false;

		return $this->block_attr['showCategories'];
	}

	// Returns a string with all attributes for categories
	public function get_category_attr(): string {
		$attr 		= array();

		// Class
		$class		= array();

		if ( 
			isset( $this->block_attr['categoryColor'] ) 
			&& $this->block_attr['categoryColorClass'] 
		) {
            $class[] = $this->block_attr['categoryColorClass'];
		}

		if ( 
			isset( $this->block_attr['categoryBackgroundColor'] ) 
			&& $this->block_attr['categoryBackgroundColorClass'] 
		) {
            $class[] = $this->block_attr['categoryBackgroundColorClass'];
		}
		
		if ( ! empty( $class ) ) {
			$attr[]	= 'class="' . implode( ' ', $class ) . '"';
		}

		// Style
		$style = array();

		if ( 
			isset( $this->block_attr['categoryColor'] ) 
			&& ! $this->block_attr['categoryColorClass'] 
		) {
			$style[] = 'color:' . $this->block_attr['categoryColor'];
		}

		if ( 
			isset( $this->block_attr['categoryBackgroundColor'] ) 
			&& ! $this->block_attr['categoryBackgroundColorClass'] 
		) {
			$style[] = 'background-color:' . $this->block_attr['categoryBackgroundColor'];
		}

		if ( ! empty( $style ) ) {
			$attr[] = 'style="' . implode( ';', $style ) . '"';
		}


		return implode( ' ', $attr );
	}

	// Returns the categories as string
	public function get_categories( string $seperator = ', ', bool $in_thumbnail = false ): string {
		if ( empty( get_the_category() ) ) return '';

		$categories = array();

		foreach ( get_the_category() as $cat ) {
			$category = '<a href="' . esc_url( get_category_link( $cat->term_id ) ) . '" ';
			$category .= 'title="' . esc_attr( sprintf( __( 'View all posts in %s', 'sv_posts' ), $cat->name ) ) . '" ';
			
			if ( $in_thumbnail ) {
				$category .= $this->get_category_attr();
			} else {
				$category .= $this->get_info_attr();
			}

			$category .= '>' . esc_html( $cat->name ) . '</a>';

			$categories[] = $category;
		}

		return implode( $seperator, $categories );
	}

	// Returns bool if the edit post link should be displayed
	public function show_edit(): bool {
		if ( ! isset( $this->block_attr['showEdit'] ) ) return false;

		return $this->block_attr['showEdit'];
	}
}